<?php

namespace WPBannerize\Models;

class WPBannerizePost
{
  use WordPressPostTrait;

  public $postType = 'wp_bannerize';

  protected $imageMimeTypes = [
    'image/gif',
    'image/png',
    'image/jpeg',
    'image/jpeg',
    'image/jpeg',
  ];

  protected $post;

  public function __get($name)
  {
    if ($this->post) {

      $metaKey = "wp_bannerize_{$name}";

      foreach ($this->post as $key => $value) {
        if ($key == $name) {
          return $this->post->{$name};
        }
      }

      return get_post_meta($this->post->ID, $metaKey, true);

    }

    return null;
  }

  public function __toString()
  {
    ob_start();

    $this->display();

    $content = ob_get_contents();
    ob_end_clean();

    return $content;
  }

  public function display()
  {
    $content = "";

    if (in_array($this->banner_mime_type, $this->imageMimeTypes)) {
      $content = $this->getImage();
    } elseif ($this->banner_mime_type == 'text/plain') {
      $content = $this->getText();
    }

    $classes = ['wp_bannerize_banner_box'];

    $terms = wp_get_post_terms($this->post->ID, 'wp_bannerize_tax');

    foreach ($terms as $term) {
      $classes[] = "wp_bannerize_category_{$term->slug}";
    }

    $classes = apply_filters('wp_bannerize_classes', $classes);

    $stack = [];

    foreach (['top', 'right', 'bottom', 'left'] as $key) {
      $value = WPBannerize()->options["Layout.{$key}"];
      if (!empty($value)) {
        $stack[] = "margin-{$key}:{$value}px";
      }
    }

    $styles = implode(';', $stack);

    ?>
    <div class="<?php echo implode(' ', $classes) ?>"
         style="<?php echo $styles ?>"
         data-title="<?php echo $this->post_title ?>"
         data-mime_type="<?php echo sanitize_title($this->banner_mime_type) ?>"
      <?php echo empty(wpbones_value($this->banner_impressions_enabled)) ? '' : ' data-impressions_enabled="true"' ?>
      <?php echo empty(wpbones_value($this->banner_clicks_enabled)) ? '' : ' data-clicks_enabled="true"' ?>
         data-banner_id="<?php echo $this->ID ?>"
         id="wpbanner-<?php echo $this->ID ?>">
      <?php echo $content ?>
    </div>
    <?php
  }

  protected function getImage()
  {
    $src   = $this->getUrl();
    $title = $this->getDescription();
    $link  = $this->banner_link;

    $target = '';
    if (!empty(wpbones_value($this->banner_target))) {
      $target = sprintf('target="%s"', $this->banner_target);
    }

    // Get width and height
    $width  = $this->getWidth();
    $height = $this->getHeight();

    // Stack
    $size = [];

    if (!empty($width)) {
      $size[] = sprintf('width="%s"', $width);
    }

    if (!empty($height)) {
      $size[] = sprintf('height="%s"', $height);
    }

    // Build size string and remove px
    $size_string = str_replace('px', '', implode(' ', $size));

    $nofollow = !empty(wpbones_value($this->banner_no_follow)) ? 'rel="nofollow"' : '';

    $html = sprintf('<img border="0" %s src="%s" alt="%s" title="%s" />', $size_string, $src, $title, $title);
    if (!empty($link)) {
      $html = sprintf('<a href="%s" %s %s>%s</a>', $link, $target, $nofollow, $html);
    }

    return $html;
  }

  public function getUrl()
  {
    return ('local' == $this->banner_type) ? $this->banner_url : $this->banner_external_url;
  }

  public function getDescription()
  {
    $description = $this->banner_description;

    return empty($description) ? $this->post_title : $description;
  }

  public function getWidth()
  {
    if (empty(wpbones_value($this->banner_width))) {
      return '';
    }

    return is_numeric($this->banner_width) ? $this->banner_width . 'px' : $this->banner_width;
  }

  public function getHeight()
  {
    if (empty(wpbones_value($this->banner_height))) {
      return '';
    }

    return is_numeric($this->banner_height) ? $this->banner_height . 'px' : $this->banner_height;
  }

  protected function getText()
  {
    // Get size
    $width  = empty(wpbones_value($this->banner_width)) ? '100%' : $this->banner_width;
    $height = empty(wpbones_value($this->banner_height)) ? '100%' : $this->banner_height;

    $html = do_shortcode($this->post_content);

    return sprintf('<div style="width:%s;height:%s">%s</div>', $width, $height, $html);
  }

  public static function all()
  {
    return get_posts(
      [
        'numberposts' => -1,
        'post_type'   => 'wp_bannerize',
      ]
    );
  }

  /**
   * Description
   *
   * @param $vars
   * @return int[]|\WP_Post[]
   */
  public static function expired($vars = [])
  {
    remove_all_filters('parse_query');

    $args = [
      'meta_query'     => self::metaQuery('expired'),
      'post_type'      => 'wp_bannerize',
      'posts_per_page' => -1,
    ];

    $args = array_merge($args, $vars);

    $posts = get_posts($args);

    return $posts;
  }

  public static function metaQuery($value)
  {
    $metaQuery = [
      'expired'   => [
        'relation' => 'AND',
        [
          'key'     => 'wp_bannerize_banner_date_expiry',
          'value'   => '',
          'compare' => '!=',
        ],
        [
          'key'     => 'wp_bannerize_banner_date_expiry',
          'value'   => time(),
          'compare' => '<',
          'type'    => 'NUMERIC',
        ],
      ],
      'scheduled' => [
        'relation' => 'AND',
        [
          'key'     => 'wp_bannerize_banner_date_from',
          'value'   => '',
          'compare' => '!=',
        ],
        [
          'key'     => 'wp_bannerize_banner_date_from',
          'value'   => time(),
          'compare' => '>',
          'type'    => 'NUMERIC',
        ],
      ],
      'timed'     => [
        'relation' => 'OR',
        [
          'key'     => 'wp_bannerize_banner_date_from',
          'value'   => '',
          'compare' => '!=',
        ],
        [
          'key'     => 'wp_bannerize_banner_date_expiry',
          'value'   => '',
          'compare' => '!=',
        ],
      ],
    ];

    return $metaQuery[$value]??[];
  }

  public static function scheduled($vars = [])
  {
    remove_all_filters('parse_query');

    $args = [
      'meta_query'     => self::metaQuery('scheduled'),
      'post_type'      => 'wp_bannerize',
      'posts_per_page' => -1,
    ];

    $args = array_merge($args, $vars);

    $posts = get_posts($args);

    return $posts;
  }

  public static function timed($vars = [])
  {
    remove_all_filters('parse_query');

    $args = [
      'meta_query'     => self::metaQuery('timed'),
      'post_type'      => 'wp_bannerize',
      'posts_per_page' => -1,
    ];

    $args = array_merge($args, $vars);

    $posts = get_posts($args);

    return $posts;
  }

  public static function where()
  {
    $instance = new self;

    if (func_num_args() > 2) {
      [$field, $cond, $value] = func_get_args();
    } elseif (func_num_args() > 1) {
      [$field, $value] = func_get_args();
      $cond = "=";
    } else {
      return null;
    }

    switch ($field) {
      case 'path':
        $instance->post = get_page_by_path($value, OBJECT, $instance->postType);
        break;
      case 'title':
        $instance->post = get_page_by_title($value, OBJECT, $instance->postType);
        break;
    }

    return $instance;
  }

  public function preview()
  {
    if (in_array($this->banner_mime_type, $this->imageMimeTypes)) {
      return WPBannerize()->view('cpt.preview-image')->with('banner', $this);
    } elseif ($this->banner_mime_type == 'text/plain') {
      return WPBannerize()->view('cpt.preview-txt')->with('banner', $this);
    }

    return WPBannerize()->view('cpt.preview-not-available')->with('banner', $this);
  }

  public function thumbnail()
  {

    if (in_array($this->banner_mime_type, $this->imageMimeTypes)) {
      return WPBannerize()->view('cpt.thumbnail-image')->with('banner', $this);
    } elseif ($this->banner_mime_type == 'text/plain') {
      return WPBannerize()->view('cpt.thumbnail-txt')->with('banner', $this);
    }

    return WPBannerize()->view('cpt.thumbnail-not-available')->with('banner', $this);
  }

  public function getSizeWithURL($url)
  {
    // Prepare return
    $size = [0, 0, 'mime' => 'text/plain'];

    // Check for txt
    if ('.txt' === $url) {
      return $size;
    }

    // Check for GD library
    if (function_exists('getimagesize')) {

      // If exists
      if (!empty($url)) {

        // Try directly
        $size = @getimagesize($url);

        // Try by CURL
        if (empty($size) && function_exists('curl_version')) {

          // Try via CURL
          $ch = curl_init();
          curl_setopt($ch, CURLOPT_URL, $url);
          curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
          curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
          curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);

          $contents = curl_exec($ch);
          curl_close($ch);

          $uri = 'data://application/octet-stream;base64,' . base64_encode($contents);

          $size = @getimagesize($uri);
        }
      }
    }

    if (empty($size['mime'])) {
      $mime = $this->getMimeTypesWithExtension($this->ext($url));
      if ($mime) {
        $size['mime'] = $mime;
      }
    }

    return $size;
  }

  protected function getMimeTypesWithExtension($ext)
  {
    $mime = [
      'gif'  => 'image/gif',
      'png'  => 'image/png',
      'jpg'  => 'image/jpeg',
      'jpe'  => 'image/jpeg',
      'jpeg' => 'image/jpeg',
      'txt'  => 'text/plain',
    ];

    if (isset($mime[$ext])) {
      return $mime[$ext];
    }

    return null;
  }

  protected function ext($filename)
  {
    $filename = strtolower(basename($filename));
    $parts    = explode('.', $filename);

    if (empty($parts)) {
      return false;
    }

    $ext = end($parts);

    return $ext;
  }

}