<?php

namespace WPBannerize\Providers;

use WPBannerize\WPBones\Support\ServiceProvider;

class WPBannerizeServiceProvider extends ServiceProvider
{

    public function register()
    {
        //plugin list
        add_action('plugin_action_links_' . WPBannerize()->pluginBasename, [$this, 'plugin_action_links'], 10, 4);

        // check for old wp bannerize table
        $result = get_option('wp_bannerize_do_import', false);

        if ($result) {
            add_action('admin_notices', [$this, 'admin_notices_import']);
        } else {

            // check for old wp bannerize table
            $result = get_option('wp_bannerize_old_table', false);

            if ($result) {
                add_action('admin_notices', [$this, 'admin_notices_table']);
            }
        }

        // Load all necessary admin bar items.
        add_action('admin_bar_menu', [$this, 'admin_bar_menu'], 100);

        WPBannerize()->css('wp-bannerize-admin-bar.min.css');

    }

    /**
     * Load all necessary admin bar items.
     *
     * This is the hook used to add, remove, or manipulate admin bar items.
     *
     * @since 3.1.0
     *
     * @param WP_Admin_Bar $wp_admin_bar WP_Admin_Bar instance, passed by reference
     */
    public function admin_bar_menu($wp_admin_bar)
    {
        // administrator only
        if (current_user_can('activate_plugins')) {

            $icon  = '<span class="ab-icon"></span>';
            $title = '<span class="ab-label" aria-hidden="true">' . __('Bannerize', WPBANNERIZE_TEXTDOMAIN) . '</span>';


            $paranetId = 'wp-bannerize-pro-parent-admin-menu';
            $nonce     = 'wp-bannerize-pro';

            $adminMenu = [
                [
                    'id' => $paranetId,
                    'title' => $icon . $title,
                    'href' => '#',
                    'meta' => [
                        'title' => __('Bannerize', WPBANNERIZE_TEXTDOMAIN),
                    ],
                ],
            ];

            $adminMenu[] = [
                'parent' => $paranetId,
                'id' => 'wp-bannerize-pro-admin-menu-add-banner',
                'title' => __('Add new Banner', WPBANNERIZE_TEXTDOMAIN),
                'href' => admin_url('post-new.php?post_type=wp_bannerize'),
                'meta' => [
                    'title' => __('Add new Banner', WPBANNERIZE_TEXTDOMAIN),
                ],
            ];

            $adminMenu[] = [
                'parent' => $paranetId,
                'id' => 'wp-bannerize-pro-admin-menu-banners',
                'title' => __('List', WPBANNERIZE_TEXTDOMAIN),
                'href' => admin_url('edit.php?post_type=wp_bannerize'),
                'meta' => [
                    'title' => __('Banners List', WPBANNERIZE_TEXTDOMAIN),
                ],
            ];

            foreach ($adminMenu as $menu) {
                $wp_admin_bar->add_node($menu);
            }

        }
    }

    public function admin_notices_table()
    {
        $import = '<a class="button button-primary" href="' . admin_url('edit.php?post_type=wp_bannerize&page=wpbannerize_import', false) . '">' . __('Click Here', WPBANNERIZE_TEXTDOMAIN) . '</a>';

        ?>
      <div class="notice notice-info is-dismissible">
        <h2>WP Bannerize</h2>
        <p><?php _e('You still got the previous version of WP Bannerize database table! Please ' . $import . ' to remove!', WPBANNERIZE_TEXTDOMAIN); ?></p>
      </div>
        <?php
    }

    public function admin_notices_import()
    {
        $import = '<a class="button button-primary" href="' . admin_url('edit.php?post_type=wp_bannerize&page=wpbannerize_import', false) . '">' . __('Import', WPBANNERIZE_TEXTDOMAIN) . '</a>';

        ?>
      <div class="notice notice-warning is-dismissible">
        <h2>WP Bannerize</h2>
        <p><?php _e('You still have to complete the import of the previous version of database table! Please, ' . $import . ' to complete the import', WPBANNERIZE_TEXTDOMAIN); ?></p>
      </div>
        <?php
    }

    public function plugin_action_links($links)
    {
        $settings = '<a href="' . admin_url('edit.php?post_type=wp_bannerize&page=wpbannerize_settings', false) . '">' . __('Settings', WPBANNERIZE_TEXTDOMAIN) . '</a>';
        array_unshift($links, $settings);

        if (get_option('wp_bannerize_do_import', false)) {
            $import = '<a href="' . admin_url('edit.php?post_type=wp_bannerize&page=wpbannerize_import', false) . '">' . __('Import', WPBANNERIZE_TEXTDOMAIN) . '</a>';
            array_unshift($links, $import);
        }

        return $links;
    }

}